@@ -1,28 +1,16 @@
-/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+/*  Copyright (C) CZ.NIC, z.s.p.o. and contributors
+ *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  For more information, see <https://www.knot-dns.cz/>
  */
 
 #include "semaphore.h"
 
 #include <assert.h>
+#include <errno.h>
 #include <limits.h>
 #include <stdlib.h>
-
-#if defined(__APPLE__)
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-#endif
+#include <stdint.h>
+#include <time.h>
 
 #define SEM_STATUS_POSIX INT_MIN
 
@@ -32,11 +20,13 @@ void knot_sem_init(knot_sem_t *sem, int value)
 	if (value < 0) {
 		goto nonposix;
 	}
+#if !defined(__APPLE__)
 	int ret = sem_init(&sem->semaphore, 1, value);
 	if (ret == 0) {
 		sem->status = SEM_STATUS_POSIX;
 		return;
 	}
+#endif
 nonposix:
 	knot_sem_init_nonposix(sem, value);
 }
@@ -63,10 +53,12 @@ void knot_sem_wait(knot_sem_t *sem)
 {
 	assert(sem != NULL);
 	if (sem->status == SEM_STATUS_POSIX) {
+#if !defined(__APPLE__)
 		int semret;
 		do {
 			semret = sem_wait(&sem->semaphore);
 		} while (semret != 0); // repeat wait as it might be interrupted by a signal
+#endif
 	} else {
 		pthread_mutex_lock(&sem->status_lock->mutex);
 		while (sem->status <= 0) {
@@ -77,6 +69,56 @@ void knot_sem_wait(knot_sem_t *sem)
 	}
 }
 
+static void timespec_now_shift(struct timespec *ts, unsigned long shift_ms)
+{
+	clock_gettime(CLOCK_REALTIME, ts);
+	uint64_t nsec = ts->tv_nsec + shift_ms * 1000000LU;
+	ts->tv_sec += nsec / 1000000000LU;
+	ts->tv_nsec = nsec % 1000000000LU;
+}
+
+static bool timespec_past(const struct timespec *ts)
+{
+	struct timespec now;
+	clock_gettime(CLOCK_REALTIME, &now);
+	return ts->tv_sec == now.tv_sec ? ts->tv_nsec <= now.tv_nsec : ts->tv_sec < now.tv_sec;
+}
+
+bool knot_sem_timedwait(knot_sem_t *sem, unsigned long ms)
+{
+	assert(sem != NULL);
+	if (ms == 0) {
+		knot_sem_wait(sem);
+		return true;
+	}
+
+	struct timespec end;
+	timespec_now_shift(&end, ms);
+	if (sem->status == SEM_STATUS_POSIX) {
+#if !defined(__APPLE__)
+		int semret;
+		do {
+			semret = sem_timedwait(&sem->semaphore, &end);
+		} while (semret != 0 && errno != ETIMEDOUT); // repeat wait as it might be interrupted by a signal
+		return (semret == 0);
+#else
+		return false;
+#endif
+	} else {
+		pthread_mutex_lock(&sem->status_lock->mutex);
+		while (sem->status <= 0 && !timespec_past(&end)) {
+			pthread_cond_timedwait(&sem->status_lock->cond, &sem->status_lock->mutex, &end);
+		}
+		if (sem->status <= 0) {
+			pthread_mutex_unlock(&sem->status_lock->mutex);
+			return false;
+		}
+		sem->status--;
+		pthread_mutex_unlock(&sem->status_lock->mutex);
+		return true;
+	}
+}
+
 void knot_sem_wait_post(knot_sem_t *sem)
 {
 	assert((sem != NULL) && (sem->status != SEM_STATUS_POSIX));
@@ -109,9 +151,11 @@ void knot_sem_post(knot_sem_t *sem)
 {
 	assert(sem != NULL);
 	if (sem->status == SEM_STATUS_POSIX) {
+#if !defined(__APPLE__)
 		int semret = sem_post(&sem->semaphore);
 		(void)semret;
 		assert(semret == 0);
+#endif
 	} else {
 		pthread_mutex_lock(&sem->status_lock->mutex);
 		sem->status++;
@@ -125,7 +169,9 @@ void knot_sem_destroy(knot_sem_t *sem)
 	assert(sem != NULL);
 	knot_sem_wait(sem); // NOTE this is questionable if the initial value was > 1
 	if (sem->status == SEM_STATUS_POSIX) {
+#if !defined(__APPLE__)
 		sem_destroy(&sem->semaphore);
+#endif
 	} else {
 		pthread_cond_destroy(&sem->status_lock->cond);
 		pthread_mutex_destroy(&sem->status_lock->mutex);
