@@ -15,7 +15,7 @@
 
 int rdb_addr_to_str(struct sockaddr_storage *addr, char *out, size_t out_len, int *port);
 
-redisContext *rdb_connect(conf_t *conf, bool require_master);
+redisContext *rdb_connect(conf_t *conf, bool require_master, const char *info);
 
 void rdb_disconnect(redisContext *rdb, bool pool_save);
 
