@@ -313,10 +313,10 @@ static const yp_item_t desc_database[] = {
 	                                                 VIRT_MEM_LIMIT(GIGA(20)), YP_SSIZE } },
 	{ C_KASP_DB,               YP_TSTR,  YP_VSTR = { "keys" } },
 	{ C_KASP_DB_MAX_SIZE,      YP_TINT,  YP_VINT = { MEGA(5), VIRT_MEM_LIMIT(GIGA(100)),
-	                                                 MEGA(500), YP_SSIZE } },
+	                                                 VIRT_MEM_LIMIT(GIGA(10)), YP_SSIZE } },
 	{ C_TIMER_DB,              YP_TSTR,  YP_VSTR = { "timers" } },
 	{ C_TIMER_DB_MAX_SIZE,     YP_TINT,  YP_VINT = { MEGA(1), VIRT_MEM_LIMIT(GIGA(100)),
-	                                                 MEGA(100), YP_SSIZE } },
+	                                                 VIRT_MEM_LIMIT(GIGA(5)), YP_SSIZE } },
 	{ C_TIMER_DB_SYNC,         YP_TOPTINT, YP_VOPTINT = { 1, UINT32_MAX, TIMER_DB_SYNC_SHUTDOWN,
 	                                                      YP_STIME, 0, timer_db_sync } },
 	{ C_CATALOG_DB,            YP_TSTR,  YP_VSTR = { "catalog" } },
