@@ -67,6 +67,10 @@ int check_listen(
 	knotd_conf_check_args_t *args
 );
 
+int check_db_listen(
+	knotd_conf_check_args_t *args
+);
+
 int check_xdp_listen(
 	knotd_conf_check_args_t *args
 );
@@ -163,10 +167,6 @@ int check_catalog_tpl(
 	knotd_conf_check_args_t *args
 );
 
-int check_rdb(
-	knotd_conf_check_args_t *args
-);
-
 int check_db_instance(
 	knotd_conf_check_args_t *args
 );
