@@ -1,17 +1,6 @@
-/*  Copyright (C) 2023 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+/*  Copyright (C) CZ.NIC, z.s.p.o. and contributors
+ *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  For more information, see <https://www.knot-dns.cz/>
  */
 
 #include <assert.h>
@@ -26,7 +15,7 @@ int event_backup(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
-	zone_backup_ctx_t *ctx = zone->backup_ctx;
+	zone_backup_ctx_t *ctx = ATOMIC_GET(zone->backup_ctx);
 	if (ctx == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -66,6 +55,6 @@ int event_backup(conf_t *conf, zone_t *zone)
 
 done:
 	ret_deinit = zone_backup_deinit(ctx);
-	zone->backup_ctx = NULL;
+	ATOMIC_SET(zone->backup_ctx, NULL);
 	return (ret != KNOT_EOK) ? ret : ret_deinit;
 }
