@@ -40,7 +40,8 @@ void event_dnssec_reschedule(conf_t *conf, zone_t *zone,
 	log_dnssec_next(zone->name, (time_t)refresh_at);
 
 	if (refresh->plan_ds_check) {
-		zone->timers.next_ds_check = now;
+		zone->timers->next_ds_check = now;
+		zone->timers->flags |= TIMERS_MODIFIED;
 	}
 
 	unsigned jitter = dnskey_sync_jitter(conf, zone);
@@ -58,6 +59,10 @@ int event_dnssec(conf_t *conf, zone_t *zone)
 {
 	assert(zone);
 
+	if (zone_contents_is_empty(zone->contents)) {
+		return KNOT_EEMPTYZONE;
+	}
+
 	zone_sign_reschedule_t resch = { 0 };
 	zone_sign_roll_flags_t r_flags = KEY_ROLL_ALLOW_ALL;
 	int sign_flags = 0;
