@@ -1,17 +1,6 @@
-/*  Copyright (C) 2024 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+/*  Copyright (C) CZ.NIC, z.s.p.o. and contributors
+ *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  For more information, see <https://www.knot-dns.cz/>
  */
 
 #pragma once
@@ -76,8 +65,9 @@ typedef struct knot_request {
 	knot_sign_context_t sign; /*!< Required for async. DDNS processing. */
 
 	const struct knot_creds *creds;
-	size_t pin_len;
-	uint8_t pin[];
+
+	const char *hostnames[RMT_MAX_PINS];
+	const uint8_t *pins[RMT_MAX_PINS];
 } knot_request_t;
 
 static inline knotd_query_proto_t flags2proto(unsigned layer_flags)
@@ -101,8 +91,8 @@ static inline knotd_query_proto_t flags2proto(unsigned layer_flags)
  * \param creds     Local (server) credentials.
  * \param edns      EDNS parameters.
  * \param tsig_key  TSIG key for authentication.
- * \param pin       Possible remote certificate PIN.
- * \param pin_len   Length of the remote certificate PIN.
+ * \param hostnames Permittable remote certificate hostnames.
+ * \param pins      Permittable remote certificate PINs.
  * \param flags     Request flags.
  *
  * \return Prepared request or NULL in case of error.
@@ -114,8 +104,8 @@ knot_request_t *knot_request_make_generic(knot_mm_t *mm,
                                           const struct knot_creds *creds,
                                           const query_edns_data_t *edns,
                                           const knot_tsig_key_t *tsig_key,
-                                          const uint8_t *pin,
-                                          size_t pin_len,
+                                          const char *const hostnames[RMT_MAX_PINS],
+                                          const uint8_t *const pins[RMT_MAX_PINS],
                                           knot_request_flag_t flags);
 
 /*!
