@@ -1,17 +1,6 @@
-/*  Copyright (C) 2024 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+/*  Copyright (C) CZ.NIC, z.s.p.o. and contributors
+ *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  For more information, see <https://www.knot-dns.cz/>
  */
 
 #include <string.h>
@@ -22,20 +11,24 @@
 #include "libknot/quic/tls.h"
 #include "contrib/conn_pool.h"
 
-int knot_tls_req_ctx_init(knot_tls_req_ctx_t *ctx, int fd,
+int knot_tls_req_ctx_init(knot_tls_req_ctx_t *ctx,
+                          int fd,
                           const struct sockaddr_storage *remote,
                           const struct sockaddr_storage *local,
                           const struct knot_creds *local_creds,
-                          const uint8_t *peer_pin, uint8_t peer_pin_len,
-                          bool *reused_fd, int io_timeout_ms)
+                          const char *const peer_hostnames[RMT_MAX_PINS],
+                          const uint8_t *const peer_pins[RMT_MAX_PINS],
+                          bool *reused_fd,
+                          int io_timeout_ms)
 {
-	struct knot_creds *creds = knot_creds_init_peer(local_creds, peer_pin, peer_pin_len);
+	struct knot_creds *creds = knot_creds_init_peer(local_creds, peer_hostnames, peer_pins);
 	if (creds == NULL) {
 		return KNOT_ENOMEM;
 	}
 
 	// Use HS = 4x IO timeout, as the RMT IO timeout is usually high.
-	ctx->ctx = knot_tls_ctx_new(creds, io_timeout_ms, 4 * io_timeout_ms, false);
+	ctx->ctx = knot_tls_ctx_new(creds, io_timeout_ms, 4 * io_timeout_ms,
+	                            KNOT_TLS_CLIENT | KNOT_TLS_DNS | KNOT_TLS_EARLY_DATA);
 	if (ctx->ctx == NULL) {
 		knot_creds_free(creds);
 		return KNOT_ENOMEM;
