@@ -13,6 +13,7 @@
 #include <sys/types.h>   // OpenBSD
 #include <netinet/tcp.h> // TCP_FASTOPEN
 #include <sys/resource.h>
+#include <urcu.h>
 
 #include "libknot/libknot.h"
 #include "libknot/yparser/ypschema.h"
@@ -40,6 +41,7 @@
 #include "contrib/conn_pool.h"
 #include "contrib/files.h"
 #include "contrib/net.h"
+#include "contrib/openbsd/siphash.h"
 #include "contrib/openbsd/strlcat.h"
 #include "contrib/os.h"
 #include "contrib/sockaddr.h"
@@ -899,6 +901,8 @@ static void rdb_process_event(redisReply *reply, knot_zonedb_t *zone_db,
 
 	switch (type) {
 	case RDB_EVENT_ZONE:
+		zone_set_flag(zone, ZONE_RDB_RELOAD);
+		// FALLTHROUGH
 	case RDB_EVENT_UPD:
 		log_zone_debug(zone->name, "rdb, event %s %s, serial %u",
 		               since, (type == RDB_EVENT_ZONE ? "zone" : "update"), serial);
@@ -960,15 +964,15 @@ static int rdb_listener_run(struct dthread *thread)
 		redisReply *reply = redisCommand(s->rdb_ctx, "XREAD BLOCK %d STREAMS %b %s",
 		                                 4000, RDB_EVENT_KEY, strlen(RDB_EVENT_KEY), since);
 		if (reply == NULL) {
-			if (thread->state & ThreadDead) {
+			if (dt_is_cancelled(thread)) {
 				break;
 			}
-			if (s->rdb_ctx->err != REDIS_OK) {
+			if (s->rdb_ctx != NULL && s->rdb_ctx->err != REDIS_OK) {
 				log_error("rdb, failed to read events (%s)", s->rdb_ctx->errstr);
 			}
 			rdb_disconnect(s->rdb_ctx, false);
 			s->rdb_ctx = NULL;
-			sleep(2);
+			sleep(1);
 			continue;
 		}
 		if (reply->type == REDIS_REPLY_NIL) {
@@ -990,16 +994,30 @@ static int rdb_listener_run(struct dthread *thread)
 }
 #endif // ENABLE_REDIS
 
+static void duration_to_str(char res[64], struct timespec *beg)
+{
+        struct timespec end = time_now();
+        double duration = time_diff_ms(beg, &end) / 1000.0;
+        if (duration >= 1.0) {
+                snprintf(res, 64, " in %0.2f seconds", duration);
+	}
+}
+
 static int timer_db_do_sync(struct dthread *thread)
 {
 	server_t *s = thread->data;
 
 	while (thread->state & ThreadActive) {
+		struct timespec beg = time_now();
+		rcu_read_lock();
 		int ret = zone_timers_write_all(&s->timerdb, s->zone_db);
+		rcu_read_unlock();
+		char dur_str[64] = "";
+		duration_to_str(dur_str, &beg);
 		if (ret == KNOT_EOK) {
-			log_info("updated persistent timer DB");
+			log_info("updated persistent timer DB%s", dur_str);
 		} else {
-			log_error("failed to update persistent timer DB (%s)", knot_strerror(ret));
+			log_error("failed to update persistent timer DB%s (%s)", dur_str, knot_strerror(ret));
 		}
 
 		if (conf()->cache.db_timer_db_sync <= 0) {
@@ -1045,7 +1063,6 @@ int server_init(server_t *server, int bg_workers)
 
 	int ret = catalog_update_init(&server->catalog_upd);
 	if (ret != KNOT_EOK) {
-		dt_stop(server->rdb_events);
 		dt_delete(&server->rdb_events);
 		worker_pool_destroy(server->workers);
 		evsched_deinit(&server->sched);
@@ -1091,11 +1108,15 @@ void server_deinit(server_t *server)
 	bool should_sync = (conf()->cache.db_timer_db_sync == TIMER_DB_SYNC_SHUTDOWN ||
 	                    conf()->cache.db_timer_db_sync > 0);
 	if (should_sync && server->zone_db != NULL) {
-		log_info("updating persistent timer DB");
+		struct timespec beg = time_now();
 		int ret = zone_timers_write_all(&server->timerdb, server->zone_db);
+		char dur_str[64] = "";
+		duration_to_str(dur_str, &beg);
 		if (ret != KNOT_EOK) {
-			log_error("failed to update persistent timer DB (%s)",
-			          knot_strerror(ret));
+			log_error("failed to update persistent timer DB%s (%s)",
+			          dur_str, knot_strerror(ret));
+		} else {
+			log_info("updated persistent timer DB%s", dur_str);
 		}
 	}
 
@@ -1182,10 +1203,8 @@ static void server_free_handler(iohandler_t *h)
 	}
 
 	/* Wait for threads to finish */
-	if (h->unit) {
-		dt_stop(h->unit);
-		dt_join(h->unit);
-	}
+	dt_stop(h->unit);
+	dt_join(h->unit);
 
 	/* Destroy worker context. */
 	dt_delete(&h->unit);
@@ -1204,7 +1223,7 @@ int server_start_answering(server_t *server)
 		return KNOT_EOK;
 	}
 
-	log_info("server started");
+	log_info("server started answering");
 	server->state |= ServerAnswering;
 	for (int proto = IO_UDP; proto <= IO_XDP; ++proto) {
 		if (server->handlers[proto].size > 0) {
@@ -1215,6 +1234,8 @@ int server_start_answering(server_t *server)
 		}
 	}
 
+	knot_zonedb_foreach(server->zone_db, zone_events_start_answering);
+
 	return KNOT_EOK;
 }
 
@@ -1556,6 +1577,7 @@ int server_reload(server_t *server, reload_t mode)
 
 		ATOMIC_SET(server->stats.tcp_io_timeout, 0);
 		ATOMIC_SET(server->stats.tcp_idle_timeout, 0);
+		ATOMIC_SET(server->stats.zone_update_error, 0);
 	}
 
 	conf_update_flag_t upd_flags = CONF_UPD_FNOFREE;
@@ -1611,10 +1633,11 @@ void server_stop(server_t *server)
 
 #ifdef ENABLE_REDIS
 	/* Interrupt and stop XREAD BLOCK loop. */
-	if (server->rdb_ctx != NULL) {
-		redisSetTimeout(server->rdb_ctx, (struct timeval){ 0, 1 });
-	}
 	dt_stop(server->rdb_events);
+	if (server->rdb_ctx != NULL) {
+		shutdown(server->rdb_ctx->fd, SHUT_RDWR);
+	}
+	dt_join(server->rdb_events);
 #endif // ENABLE_REDIS
 
 	/* Stop scheduler. */
@@ -1623,6 +1646,7 @@ void server_stop(server_t *server)
 	server->state |= ServerShutting;
 	/* Stop timer DB syncing thread */
 	dt_stop(server->timerdb_sync);
+	dt_join(server->timerdb_sync);
 	/* Interrupt background workers. */
 	worker_pool_stop(server->workers);
 
@@ -1707,7 +1731,7 @@ static int reconfigure_timer_db(conf_t *conf, server_t *server)
 		}
 	} else if (!should_sync && exists_sync) {
 		dt_stop(server->timerdb_sync);
-		dt_delete(&server->timerdb_sync);
+		dt_join(server->timerdb_sync);
 	}
 
 	return ret;
@@ -1720,6 +1744,7 @@ static void free_sess_ticket(intptr_t ptr)
 	}
 }
 
+#ifdef ENABLE_REDIS
 static void free_redis_conn(intptr_t ptr)
 {
 	if (ptr != CONN_POOL_FD_INVALID) {
@@ -1732,6 +1757,16 @@ static bool invalid_redis_conn(intptr_t ptr)
 	return ptr == CONN_POOL_FD_INVALID || !zone_redis_ping((void *)ptr);
 }
 
+static uint64_t db_listen_hash(conf_val_t *val)
+{
+	SIPHASH_CTX ctx;
+	SIPHASH_KEY key = { 0 };
+	SipHash24_Init(&ctx, &key);
+	SipHash24_Update(&ctx, val->blob, val->blob_len);
+	return SipHash24_End(&ctx);
+}
+#endif // ENABLE_REDIS
+
 static int reconfigure_remote_pool(conf_t *conf, server_t *server)
 {
 	conf_val_t val = conf_get(conf, C_SRV, C_RMT_POOL_LIMIT);
@@ -1771,16 +1806,31 @@ static int reconfigure_remote_pool(conf_t *conf, server_t *server)
 		hash = curr_hash;
 	}
 
+#ifdef ENABLE_REDIS
 	val = conf_get(conf, C_DB, C_ZONE_DB_LISTEN);
-	if (global_redis_pool == NULL && val.code == KNOT_EOK) {
-		size_t bg_wrkrs = conf_bg_threads(conf);
-		conn_pool_t *new_pool = conn_pool_init(bg_wrkrs, REDIS_CONN_POOL_TIMEOUT,
-		                                       free_redis_conn, invalid_redis_conn);
-		if (new_pool == NULL) {
-			return KNOT_ENOMEM;
+	if (val.code == KNOT_EOK) {
+		static uint64_t hash = 0;
+		uint64_t curr_hash = db_listen_hash(&val);
+
+		if (global_redis_pool == NULL) {
+			size_t bg_wrkrs = conf_bg_threads(conf);
+			conn_pool_t *new_pool = conn_pool_init(bg_wrkrs + 1, REDIS_CONN_POOL_TIMEOUT,
+			                                       free_redis_conn, invalid_redis_conn);
+			if (new_pool == NULL) {
+				return KNOT_ENOMEM;
+			}
+			global_redis_pool = new_pool;
 		}
-		global_redis_pool = new_pool;
+		if (hash != curr_hash) {
+			conn_pool_purge(global_redis_pool);
+			if (server->rdb_ctx != NULL) {
+				// Interrupt XREAD BLOCK and reconnect events.
+				shutdown(server->rdb_ctx->fd, SHUT_RDWR);
+			}
+		}
+		hash = curr_hash;
 	}
+#endif // ENABLE_REDIS
 
 	val = conf_get(conf, C_SRV, C_RMT_RETRY_DELAY);
 	int delay_ms = conf_int(&val);
@@ -1852,7 +1902,7 @@ int server_reconfigure(conf_t *conf, server_t *server)
 
 	/* Reconfigure Timer DB. */
 	if ((ret = reconfigure_timer_db(conf, server)) != KNOT_EOK) {
-		log_error("failed to reconfigure Timer DB (%s)",
+		log_error("failed to reconfigure timer DB (%s)",
 		          knot_strerror(ret));
 	}
 
