@@ -870,24 +870,6 @@ static int update_catalog(conf_t *conf, zone_update_t *update)
 	return ret;
 }
 
-typedef struct {
-	pthread_mutex_t lock;
-	size_t counter;
-} counter_reach_t;
-
-static bool counter_reach(counter_reach_t *counter, size_t increment, size_t limit)
-{
-	bool reach = false;
-	pthread_mutex_lock(&counter->lock);
-	counter->counter += increment;
-	if (counter->counter >= limit) {
-		counter->counter = 0;
-		reach = true;
-	}
-	pthread_mutex_unlock(&counter->lock);
-	return reach;
-}
-
 /*! \brief Struct for what needs to be cleared after RCU.
  *
  * This can't be zone_update_t structure as this might be already freed at that time.
@@ -904,12 +886,38 @@ typedef struct {
 	size_t new_cont_size;
 } update_clear_ctx_t;
 
+static void call_mem_trim(update_clear_ctx_t *ctx)
+{
+	static pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+
+	static bool running = false;
+	static size_t counter = 0;
+	static knot_millis_t next = 0;
+
+	knot_millis_t now = knot_millis_now();
+	pthread_mutex_lock(&lock);
+	counter += ctx->new_cont_size;
+	if (running || counter < UPDATE_MEMTRIM_AT || now < next) {
+		pthread_mutex_unlock(&lock);
+		return;
+	}
+	running = true;
+	pthread_mutex_unlock(&lock);
+
+	mem_trim();
+	knot_millis_t diff = knot_millis_now() - now;
+
+	pthread_mutex_lock(&lock);
+	running = false;
+	counter = 0;
+	next = now + 2 * diff;
+	pthread_mutex_unlock(&lock);
+}
+
 static void *update_clear_retry(void *arg);
 
 static void update_clear(struct rcu_head *param)
 {
-	static counter_reach_t counter = { PTHREAD_MUTEX_INITIALIZER, 0 };
-
 	update_clear_ctx_t *ctx = (update_clear_ctx_t *)param;
 
 	if (ctx->free_contents != NULL) {
@@ -929,9 +937,7 @@ static void update_clear(struct rcu_head *param)
 	apply_cleanup(ctx->cleanup_apply);
 	free(ctx->cleanup_apply);
 
-	if (counter_reach(&counter, ctx->new_cont_size, UPDATE_MEMTRIM_AT)) {
-		mem_trim();
-	}
+	call_mem_trim(ctx);
 
 	free(ctx);
 }
@@ -987,8 +993,9 @@ int zone_update_verify_digest(conf_t *conf, zone_update_t *update)
 	if (!conf_bool(&val)) {
 		return KNOT_EOK;
 	}
+	val = conf_zone_get(conf, C_DNSSEC_SIGNING, update->zone->name);
 
-	int ret = zone_contents_digest_verify(update->new_cont);
+	int ret = zone_contents_digest_verify(update->new_cont, conf_bool(&val));
 	if (ret != KNOT_EOK) {
 		log_zone_error(update->zone->name, "ZONEMD, verification failed (%s)",
 		               knot_strerror(ret));
@@ -1143,7 +1150,7 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		ret = commit_full(conf, update);
 	}
 	if (ret != KNOT_EOK) {
-		return ret;
+		goto error;
 	}
 
 	conf_val_t thr = conf_zone_get(conf, C_ADJUST_THR, update->zone->name);
@@ -1154,7 +1161,7 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 	}
 	if (ret != KNOT_EOK) {
 		discard_adds_tree(update);
-		return ret;
+		goto error;
 	}
 
 	/* Check the zone size. */
@@ -1163,7 +1170,8 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 
 	if (update->new_cont->size > size_limit) {
 		discard_adds_tree(update);
-		return KNOT_EZONESIZE;
+		ret = KNOT_EZONESIZE;
+		goto error;
 	}
 
 	val = conf_zone_get(conf, C_DNSSEC_VALIDATION, update->zone->name);
@@ -1176,7 +1184,7 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		ret = knot_dnssec_validate_zone(update, &val_conf);
 		if (ret != KNOT_EOK) {
 			discard_adds_tree(update);
-			return ret;
+			goto error;
 		}
 	}
 
@@ -1185,7 +1193,7 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		ret = zone_update_external(conf, update, &val);
 		if (ret != KNOT_EOK) {
 			discard_adds_tree(update);
-			return ret;
+			goto error;
 		}
 	}
 
@@ -1193,21 +1201,21 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 	if (ret != KNOT_EOK) {
 		log_zone_error(update->zone->name, "failed to process catalog zone (%s)", knot_strerror(ret));
 		discard_adds_tree(update);
-		return ret;
+		goto error;
 	}
 
 	ret = commit_redis(conf, update);
 	if (ret != KNOT_EOK) {
 		log_zone_error(update->zone->name, "zone database update failed (%s)", knot_strerror(ret));
 		discard_adds_tree(update);
-		return ret;
+		goto error;
 	}
 
 	ret = commit_journal(conf, update);
 	if (ret != KNOT_EOK) {
 		log_zone_error(update->zone->name, "journal update failed (%s)", knot_strerror(ret));
 		discard_adds_tree(update);
-		return ret;
+		goto error;
 	}
 
 	if (dnssec) {
@@ -1253,13 +1261,20 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 	}
 
 	if (conf->cache.srv_dbus_event & DBUS_EVENT_ZONE_UPDATED) {
-		dbus_emit_zone_updated(update->zone->name,
+		dbus_emit_zone_updated(update->zone->name, true,
 		                       zone_contents_serial(update->zone->contents));
 	}
 
 	memset(update, 0, sizeof(*update));
 
 	return KNOT_EOK;
+error:
+	ATOMIC_ADD(update->zone->server->stats.zone_update_error, 1);
+	if (conf->cache.srv_dbus_event & DBUS_EVENT_ZONE_UPDATED) {
+		dbus_emit_zone_updated(update->zone->name, false, 0);
+	}
+
+	return ret;
 }
 
 bool zone_update_no_change(zone_update_t *update)
