@@ -8,8 +8,6 @@
 #include "contrib/wire_ctx.h"
 #include "knot/zone/zonedb.h"
 
-#include <urcu.h>
-
 /*
  * # Timer database
  *
@@ -210,10 +208,10 @@ int zone_timers_write(knot_lmdb_db_t *db, const knot_dname_t *zone,
 
 static void txn_zone_write(zone_t *z, knot_lmdb_txn_t *txn)
 {
-	rcu_read_lock();
-	zone_timers_t *t = z->timers_static;
-	txn_write_timers(txn, z->name, t);
-	rcu_read_unlock();
+	if (z->started) {
+                zone_timers_t *t = z->timers_static;
+                txn_write_timers(txn, z->name, t);
+	}
 }
 
 int zone_timers_write_all(knot_lmdb_db_t *db, knot_zonedb_t *zonedb)
