@@ -1,17 +1,6 @@
-/*  Copyright (C) 2024 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+/*  Copyright (C) CZ.NIC, z.s.p.o. and contributors
+ *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  For more information, see <https://www.knot-dns.cz/>
  */
 
 #include <assert.h>
@@ -143,8 +132,9 @@ static ngtcp2_conn *get_conn(ngtcp2_crypto_conn_ref *conn_ref)
 static int tls_init_conn_session(knot_quic_conn_t *conn, bool server)
 {
 	int ret = knot_tls_session(&conn->tls_session, conn->quic_table->creds,
-	                           conn->quic_table->priority, true,
-	                           true, server);
+	                           conn->quic_table->priority,
+	                           (server ? KNOT_TLS_SERVER : KNOT_TLS_CLIENT) |
+	                           KNOT_TLS_QUIC | KNOT_TLS_DNS | KNOT_TLS_EARLY_DATA);
 	if (ret != KNOT_EOK) {
 		return TLS_CALLBACK_ERR;
 	}
@@ -299,8 +289,10 @@ static int handshake_completed_cb(ngtcp2_conn *conn, void *user_data)
 	ctx->flags |= KNOT_QUIC_CONN_HANDSHAKE_DONE;
 
 	if (!ngtcp2_conn_is_server(conn)) {
-		return knot_tls_pin_check(ctx->tls_session, ctx->quic_table->creds)
-		       == KNOT_EOK ? 0 : NGTCP2_ERR_CALLBACK_FAILURE;
+		return knot_tls_pin_check(ctx->tls_session, ctx->quic_table->creds) == KNOT_EOK
+		       && knot_tls_cert_check(ctx->tls_session, ctx->quic_table->creds) == KNOT_EOK
+		               ? 0
+		               : NGTCP2_ERR_CALLBACK_FAILURE;
 	}
 
 	if (gnutls_session_ticket_send(ctx->tls_session, 1, 0) != GNUTLS_E_SUCCESS) {
@@ -737,7 +729,7 @@ int knot_quic_handle(knot_quic_table_t *table, knot_quic_reply_t *reply,
 		goto finish;
 	} else if (ngtcp2_err_is_fatal(ret)) { // connection doomed
 		if (ret == NGTCP2_ERR_CALLBACK_FAILURE) {
-			ret = KNOT_EBADCERTKEY;
+			ret = KNOT_EBADCERT;
 		} else {
 			ret = KNOT_ECONN;
 		}
