@@ -1,17 +1,6 @@
-/*  Copyright (C) 2024 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <https://www.gnu.org/licenses/>.
+/*  Copyright (C) CZ.NIC, z.s.p.o. and contributors
+ *  SPDX-License-Identifier: GPL-2.0-or-later
+ *  For more information, see <https://www.knot-dns.cz/>
  */
 
 #include <assert.h>
@@ -60,6 +49,7 @@ knot_quic_table_t *knot_quic_table_new(size_t max_conns, size_t max_ibufs, size_
 	res->max_conns = max_conns;
 	res->ibufs_max = max_ibufs;
 	res->obufs_max = max_obufs;
+	ATOMIC_INIT(res->obufs_size, 0);
 	res->udp_payload_limit = udp_payload;
 
 	int ret = gnutls_priority_init2(&res->priority, knot_tls_priority(false), NULL,
@@ -99,7 +89,9 @@ void knot_quic_table_free(knot_quic_table_t *table)
 		assert(table->usage == 0);
 		assert(table->pointers == 0);
 		assert(table->ibufs_size == 0);
-		assert(table->obufs_size == 0);
+		assert(ATOMIC_GET(table->obufs_size) == 0);
+
+		ATOMIC_DEINIT(table->obufs_size);
 
 		gnutls_priority_deinit(table->priority);
 		heap_deinit(table->expiry_heap);
@@ -134,7 +126,7 @@ void knot_quic_table_sweep(knot_quic_table_t *table, struct knot_quic_reply *swe
 			knot_sweep_stats_incr(stats, KNOT_SWEEP_CTR_LIMIT_CONN);
 			send_excessive_load(c, sweep_reply, table);
 			knot_quic_table_rem(c, table);
-		} else if (table->obufs_size > table->obufs_max) {
+		} else if (ATOMIC_GET(table->obufs_size) > table->obufs_max) {
 			knot_sweep_stats_incr(stats, KNOT_SWEEP_CTR_LIMIT_OBUF);
 			send_excessive_load(c, sweep_reply, table);
 			knot_quic_table_rem(c, table);
