@@ -188,7 +188,7 @@ static bool zone_txn_is_open(RedisModuleCtx *ctx, const arg_dname_t *origin, con
 }
 
 static void commit_event(RedisModuleCtx *ctx, rdb_event_t type, const arg_dname_t *origin,
-                         uint8_t instance, uint32_t serial)
+                         uint8_t instance, uint32_t serial, RedisModuleStreamID *stream_id)
 {
 	RedisModuleString *keyname = RedisModule_CreateString(ctx, RDB_EVENT_KEY, strlen(RDB_EVENT_KEY));
 	RedisModuleKey *stream_key = RedisModule_OpenKey(ctx, keyname, REDISMODULE_READ | REDISMODULE_WRITE);
@@ -213,8 +213,11 @@ static void commit_event(RedisModuleCtx *ctx, rdb_event_t type, const arg_dname_
 		NULL,
 	};
 
-	RedisModuleStreamID ts;
-	int ret = RedisModule_StreamAdd(stream_key, REDISMODULE_STREAM_ADD_AUTOID, &ts, events, 4);
+	int flags = 0;
+	if (stream_id->ms == 0) {
+		flags = REDISMODULE_STREAM_ADD_AUTOID;
+	}
+	int ret = RedisModule_StreamAdd(stream_key, flags, stream_id, events, 4);
 
 	for (RedisModuleString **event = events; *event != NULL; event++) {
 		RedisModule_FreeString(ctx, *event);
@@ -231,11 +234,12 @@ static void commit_event(RedisModuleCtx *ctx, rdb_event_t type, const arg_dname_
 		return;
 	}
 
-	ts.ms -= 1000LLU * rdb_event_age;
-	ts.seq = 0;
+	RedisModuleStreamID trim_id = {
+		.ms = stream_id->ms - 1000LLU * rdb_event_age
+	};
 
 	// NOTE Trimming with REDISMODULE_STREAM_TRIM_APPROX improves preformance
-	long long removed_cnt = RedisModule_StreamTrimByID(stream_key, REDISMODULE_STREAM_TRIM_APPROX, &ts);
+	long long removed_cnt = RedisModule_StreamTrimByID(stream_key, REDISMODULE_STREAM_TRIM_APPROX, &trim_id);
 	if (removed_cnt) {
 		RedisModule_Log(ctx, REDISMODULE_LOGLEVEL_NOTICE, "stream cleanup %lld old events", removed_cnt);
 	}
@@ -668,6 +672,7 @@ static void zone_begin_txt_format(RedisModuleCtx *ctx, const arg_dname_t *origin
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
 	} else {
+		RedisModule_ReplicateVerbatim(ctx);
 		RedisModule_ReplyWithLongLong(ctx, serialize_transaction(txn));
 	}
 }
@@ -678,6 +683,7 @@ static void zone_begin_bin_format(RedisModuleCtx *ctx, const arg_dname_t *origin
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
 	} else {
+		RedisModule_ReplicateVerbatim(ctx);
 		RedisModule_ReplyWithStringBuffer(ctx, (const char *)txn, sizeof(*txn));
 	}
 }
@@ -716,10 +722,10 @@ static void upd_begin_txt_format(RedisModuleCtx *ctx, const arg_dname_t *origin,
 	exception_t e = upd_begin(ctx, origin, txn);
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
-		return;
+	} else {
+		RedisModule_ReplicateVerbatim(ctx);
+		RedisModule_ReplyWithLongLong(ctx, serialize_transaction(txn));
 	}
-
-	RedisModule_ReplyWithLongLong(ctx, serialize_transaction(txn));
 }
 
 static void upd_begin_bin_format(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t *txn)
@@ -727,10 +733,10 @@ static void upd_begin_bin_format(RedisModuleCtx *ctx, const arg_dname_t *origin,
 	exception_t e = upd_begin(ctx, origin, txn);
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
-		return;
+	} else {
+		RedisModule_ReplicateVerbatim(ctx);
+		RedisModule_ReplyWithStringBuffer(ctx, (const char *)txn, sizeof(*txn));
 	}
-
-	RedisModule_ReplyWithStringBuffer(ctx, (const char *)txn, sizeof(*txn));
 }
 
 static upd_meta_k upd_meta_get_when_open(RedisModuleCtx *ctx, const arg_dname_t *origin,
@@ -892,6 +898,7 @@ static void upd_add_bin_format(RedisModuleCtx *ctx, const arg_dname_t *origin,
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
 	} else {
+		RedisModule_ReplicateVerbatim(ctx);
 		RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 	}
 }
@@ -905,6 +912,7 @@ static void upd_remove_bin_format(RedisModuleCtx *ctx, const arg_dname_t *origin
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
 	} else {
+		RedisModule_ReplicateVerbatim(ctx);
 		RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 	}
 }
@@ -1024,6 +1032,7 @@ static void zone_store_bin_format(RedisModuleCtx *ctx, const arg_dname_t *origin
 		return;
 	}
 
+	RedisModule_ReplicateVerbatim(ctx);
 	RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 }
 
@@ -1043,6 +1052,7 @@ static void run_scanner(scanner_ctx_t *s_ctx, const arg_dname_t *origin,
 	}
 	zs_deinit(&s);
 
+	RedisModule_ReplicateVerbatim(s_ctx->ctx);
 	RedisModule_ReplyWithSimpleString(s_ctx->ctx, RDB_RETURN_OK);
 }
 
@@ -1288,6 +1298,7 @@ static void zone_purge_v(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, RDB_EEVENT);
 	} else {
+		RedisModule_ReplicateVerbatim(ctx);
 		RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 	}
 }
@@ -1296,43 +1307,33 @@ typedef struct {
 	RedisModuleCtx *ctx;
 	size_t count;
 	int ret;
-	bool txt;
-	bool instances;
-} scan_ctx;
+	uint8_t instance; // Used by zone_info
+	bool txt;         // Used by zone_list
+	bool instances;   // Used by zone_list
+} scan_ctx_t;
 
 static void zone_list_cb(RedisModuleKey *key, RedisModuleString *zone_name,
                          RedisModuleString *mask, void *privdata)
 {
-	scan_ctx *sctx = privdata;
+	scan_ctx_t *sctx = privdata;
+
 	if (sctx->txt) {
+		char buf[KNOT_DNAME_TXT_MAXLEN + INSTANCE_MAX * 3];
+
 		size_t len;
 		const char *dname = RedisModule_StringPtrLen(zone_name, &len);
-		char buf[KNOT_DNAME_TXT_MAXLEN + TXN_MAX * 3];
-		if (knot_dname_to_str(buf, (knot_dname_t *)dname, sizeof(buf)) == NULL) {
-			sctx->ret = KNOT_EMALF;
-			RedisModule_ReplyWithError(sctx->ctx, RDB_ECORRUPTED);
-			++(sctx->count);
-			return;
-		}
-		char *buf_ptr = buf + knot_dname_size((knot_dname_t *)dname) - 1;
-		size_t mask_len = 0;
-		const uint8_t *mask_p = (const uint8_t *)RedisModule_StringPtrLen(mask, &mask_len);
-		if (mask_len != sizeof(uint8_t)) {
-			sctx->ret = KNOT_EMALF;
-			RedisModule_ReplyWithError(sctx->ctx, RDB_ECORRUPTED);
-			++(sctx->count);
-			return;
-		}
+		RedisModule_Assert(knot_dname_to_str(buf, (knot_dname_t *)dname, sizeof(buf)) != NULL);
+
 		if (sctx->instances) {
-			int count = 0;
-			for (int it = 0; it < TXN_MAX; ++it) {
-				const char separator = (count) ? ',' : ':';
-				if ((*mask_p) & (1 << it)) {
-					*(buf_ptr++) = separator;
-					*(buf_ptr++) = ' ';
-					*(buf_ptr++) = '0' + it + 1;
-					*buf_ptr = '\0';
-					count++;
+			const uint8_t *mask_p = (const uint8_t *)RedisModule_StringPtrLen(mask, &len);
+			RedisModule_Assert(len == sizeof(uint8_t));
+
+			bool first = true;
+			for (unsigned inst = 1; inst <= INSTANCE_MAX; ++inst) {
+				if ((*mask_p) & (1 << (inst - 1))) {
+					char item[] = { first ? ':' : ',', ' ', '0' + inst, '\0' };
+					strlcat(buf, item, sizeof(buf));
+					first = false;
 				}
 			}
 		}
@@ -1346,29 +1347,199 @@ static void zone_list_cb(RedisModuleKey *key, RedisModuleString *zone_name,
 			RedisModule_ReplyWithString(sctx->ctx, zone_name);
 		}
 	}
+
 	++(sctx->count);
 }
 
 static void zone_list(RedisModuleCtx *ctx, bool instances, bool txt)
 {
-	RedisModuleKey *zones_index = get_zones_index(ctx, REDISMODULE_READ);
-	if (zones_index == NULL) {
-		RedisModule_ReplyWithEmptyArray(ctx);
-		return;
-	}
-
-	RedisModule_ReplyWithArray(ctx, REDISMODULE_POSTPONED_ARRAY_LEN);
-
-	scan_ctx sctx = {
+	scan_ctx_t sctx = {
 		.ctx = ctx,
 		.txt = txt,
 		.instances = instances
 	};
+
+	RedisModuleKey *zones_index = get_zones_index(ctx, REDISMODULE_READ);
+	if (zones_index == NULL) {
+		RedisModule_ReplyWithError(ctx, RDB_EALLOC);
+		return;
+	}
 	RedisModuleScanCursor *cursor = RedisModule_ScanCursorCreate();
+	if (cursor == NULL) {
+		RedisModule_CloseKey(zones_index);
+		RedisModule_ReplyWithError(ctx, RDB_EALLOC);
+		return;
+	}
+
+	RedisModule_ReplyWithArray(ctx, REDISMODULE_POSTPONED_ARRAY_LEN);
 	while (RedisModule_ScanKey(zones_index, cursor, zone_list_cb, &sctx) && sctx.ret == KNOT_EOK);
 	RedisModule_ReplySetArrayLength(ctx, sctx.count);
-	RedisModule_CloseKey(zones_index);
+
 	RedisModule_ScanCursorDestroy(cursor);
+	RedisModule_CloseKey(zones_index);
+}
+
+exception_t zone_info_serial(RedisModuleCtx *ctx, size_t *counter, const arg_dname_t *origin,
+                             const rdb_txn_t *txn, const uint32_t serial_end, const uint32_t serial)
+{
+	index_k upd_index_key = get_commited_upd_index(ctx, origin, txn, serial, REDISMODULE_READ);
+	if (upd_index_key == NULL) {
+		return_ok;
+	}
+
+	uint32_t serial_next = 0;
+	exception_t e = index_soa_serial(ctx, upd_index_key, true, &serial_next);
+	if (e.ret != KNOT_EOK) {
+		RedisModule_CloseKey(upd_index_key);
+		raise(e);
+	}
+
+	if (serial_next != serial_end) {
+		e = zone_info_serial(ctx, counter, origin, txn, serial_end, serial_next);
+		if (e.ret != KNOT_EOK) {
+			RedisModule_CloseKey(upd_index_key);
+			raise(e);
+		}
+	}
+
+	char buf[64];
+	(void)snprintf(buf, sizeof(buf), "update: %u -> %u", serial_next, serial);
+	RedisModule_ReplyWithCString(ctx, buf);
+	*counter += 1;
+
+	RedisModule_CloseKey(upd_index_key);
+
+	return_ok;
+}
+
+static void zone_info_serials(RedisModuleCtx *ctx, arg_dname_t *origin, rdb_txn_t *txn)
+{
+	if (set_active_transaction(ctx, origin, txn) != KNOT_EOK) {
+		RedisModule_ReplyWithError(ctx, RDB_EZONE);
+		return;
+	}
+
+	RedisModuleString *soa_rrset_keyname = rrset_keyname_construct(ctx, origin, txn, origin, KNOT_RRTYPE_SOA);
+	rrset_k soa_rrset_key = RedisModule_OpenKey(ctx, soa_rrset_keyname, REDISMODULE_READ);
+	RedisModule_FreeString(ctx, soa_rrset_keyname);
+	rrset_v *rrset = RedisModule_ModuleTypeGetValue(soa_rrset_key);
+	if (rrset == NULL) {
+		RedisModule_CloseKey(soa_rrset_key);
+		RedisModule_ReplyWithError(ctx, RDB_ENOSOA);
+		return;
+	}
+	uint32_t serial_it = knot_soa_serial(rrset->rrs.rdata);
+	RedisModule_CloseKey(soa_rrset_key);
+
+	RedisModule_ReplyWithArray(ctx, 3);
+	char buf[64];
+	(void)snprintf(buf, sizeof(buf), "instance: %u", txn->instance);
+	RedisModule_ReplyWithCString(ctx, buf);
+	(void)snprintf(buf, sizeof(buf), "serial: %u", serial_it);
+	RedisModule_ReplyWithCString(ctx, buf);
+
+	size_t upd_count = 0;
+	RedisModule_ReplyWithArray(ctx, REDISMODULE_POSTPONED_ARRAY_LEN);
+	exception_t e = zone_info_serial(ctx, &upd_count, origin, txn, serial_it, serial_it);
+	if (e.ret != KNOT_EOK && e.what != NULL) {
+		RedisModule_ReplyWithError(ctx, e.what);
+		upd_count++;
+	}
+	RedisModule_ReplySetArrayLength(ctx, upd_count);
+}
+
+static void zone_info_cb(RedisModuleKey *key, RedisModuleString *zone_name,
+                         RedisModuleString *mask, void *privdata)
+{
+	scan_ctx_t *sctx = privdata;
+
+	char buf[KNOT_DNAME_TXT_MAXLEN];
+
+	size_t len;
+	const char *dname = RedisModule_StringPtrLen(zone_name, &len);
+	arg_dname_t origin = { .data = (uint8_t *)dname, .len = len };
+	RedisModule_Assert(knot_dname_to_str(buf, (knot_dname_t *)dname, sizeof(buf)) != NULL);
+
+	const uint8_t *mask_p = (const uint8_t *)RedisModule_StringPtrLen(mask, &len);
+	RedisModule_Assert(len == sizeof(uint8_t));
+
+	RedisModule_ReplyWithArray(sctx->ctx, 2);
+	RedisModule_ReplyWithCString(sctx->ctx, buf);
+
+	RedisModule_ReplyWithArray(sctx->ctx, REDISMODULE_POSTPONED_ARRAY_LEN);
+	size_t inst_count = 0;
+	if (sctx->instance != 0) {
+		if ((*mask_p) & (1 << (sctx->instance - 1))) {
+			rdb_txn_t txn = {
+				.instance = sctx->instance,
+				.id = TXN_ID_ACTIVE
+			};
+			zone_info_serials(sctx->ctx, &origin, &txn);
+			++inst_count;
+		}
+	} else {
+		for (unsigned inst = 1; inst <= INSTANCE_MAX; ++inst) {
+			if ((*mask_p) & (1 << (inst - 1))) {
+				rdb_txn_t txn = {
+					.instance = inst,
+					.id = TXN_ID_ACTIVE
+				};
+				zone_info_serials(sctx->ctx, &origin, &txn);
+				++inst_count;
+			}
+		}
+	}
+	RedisModule_ReplySetArrayLength(sctx->ctx, inst_count);
+
+	++(sctx->count);
+}
+
+static void zone_info(RedisModuleCtx *ctx, arg_dname_t *origin, rdb_txn_t *txn)
+{
+	scan_ctx_t sctx = {
+		.ctx = ctx,
+		.instance = (txn != NULL) ? txn->instance : 0
+	};
+
+	RedisModuleKey *zones_index = get_zones_index(ctx, REDISMODULE_READ);
+	if (zones_index == NULL) {
+		RedisModule_ReplyWithError(ctx, RDB_EALLOC);
+		return;
+	}
+
+	if (origin != NULL) {
+		RedisModuleString *origin_str = RedisModule_CreateString(ctx, (const char *)origin->data, origin->len);
+		if (origin_str == NULL) {
+			RedisModule_CloseKey(zones_index);
+			RedisModule_ReplyWithError(ctx, RDB_EALLOC);
+			return;
+		}
+
+		RedisModuleString *value;
+		if (RedisModule_HashGet(zones_index, REDISMODULE_HASH_NONE, origin_str, &value, NULL) != REDISMODULE_OK) {
+			RedisModule_FreeString(ctx, origin_str);
+			RedisModule_CloseKey(zones_index);
+			RedisModule_ReplyWithError(ctx, RDB_ECORRUPTED);
+			return;
+		}
+		zone_info_cb(zones_index, origin_str, value, &sctx);
+		RedisModule_FreeString(ctx, value);
+		RedisModule_FreeString(ctx, origin_str);
+	} else {
+		RedisModuleScanCursor *cursor = RedisModule_ScanCursorCreate();
+		if (cursor == NULL) {
+			RedisModule_CloseKey(zones_index);
+			RedisModule_ReplyWithError(ctx, RDB_EALLOC);
+			return;
+		}
+
+		RedisModule_ReplyWithArray(ctx, REDISMODULE_POSTPONED_ARRAY_LEN);
+		while (RedisModule_ScanKey(zones_index, cursor, zone_info_cb, &sctx) && sctx.ret == KNOT_EOK);
+		RedisModule_ReplySetArrayLength(ctx, sctx.count);
+
+		RedisModule_ScanCursorDestroy(cursor);
+	}
+	RedisModule_CloseKey(zones_index);
 }
 
 static exception_t zone_meta_active_exchange(RedisModuleCtx *ctx, zone_meta_k key,
@@ -1394,7 +1565,8 @@ static exception_t zone_meta_active_exchange(RedisModuleCtx *ctx, zone_meta_k ke
 	raise(e);
 }
 
-static void zone_commit(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t *txn)
+static void zone_commit(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t *txn,
+                        RedisModuleStreamID *stream_id)
 {
 	zone_meta_k meta_key = zone_meta_get_when_open(ctx, origin, txn, REDISMODULE_READ | REDISMODULE_WRITE);
 	if (meta_key == NULL) {
@@ -1461,7 +1633,13 @@ static void zone_commit(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_
 	RedisModule_FreeString(ctx, value);
 	RedisModule_CloseKey(zones_index);
 
-	commit_event(ctx, RDB_EVENT_ZONE, origin, txn->instance, serial);
+	commit_event(ctx, RDB_EVENT_ZONE, origin, txn->instance, serial, stream_id);
+
+	// Replicate with explicit ID so replicas use the same ID.
+	RedisModuleStreamID wire_id = WIRE_STREAM_ID(stream_id);
+	RedisModule_Replicate(ctx, RDB_CMD_ZONE_COMMIT, "bbb", (char *)origin->data,
+	                      origin->len, txn, sizeof(*txn), &wire_id, sizeof(wire_id));
+
 	RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 }
 
@@ -1485,6 +1663,7 @@ static void zone_abort(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t
 
 	int ret = delete_zone_index(ctx, origin, txn);
 	if (ret == KNOT_EOK) {
+		RedisModule_ReplicateVerbatim(ctx);
 		RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 	} else if (ret == KNOT_EEXIST) {
 		RedisModule_ReplyWithError(ctx, RDB_EZONE);
@@ -1734,6 +1913,7 @@ static void upd_abort_v(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
 	} else {
+		RedisModule_ReplicateVerbatim(ctx);
 		RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 	}
 }
@@ -1861,7 +2041,8 @@ static exception_t upd_trim_history(RedisModuleCtx *ctx, const arg_dname_t *orig
 	return_ok;
 }
 
-static void upd_commit(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t *upd_txn)
+static void upd_commit(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t *upd_txn,
+                       RedisModuleStreamID *stream_id)
 {
 	upd_meta_k meta_key = upd_meta_get_when_open(ctx, origin, upd_txn, REDISMODULE_READ | REDISMODULE_WRITE);
 	if (meta_key == NULL) {
@@ -2049,10 +2230,15 @@ static void upd_commit(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t
 	upd_meta_unlock(ctx, meta_key, upd_txn->id);
 	RedisModule_CloseKey(meta_key);
 
-	commit_event(ctx, RDB_EVENT_UPD, origin, upd_txn->instance, serial_new);
+	commit_event(ctx, RDB_EVENT_UPD, origin, upd_txn->instance, serial_new, stream_id);
+
 	if (e.ret != KNOT_EOK) {
 		RedisModule_ReplyWithError(ctx, e.what);
 	} else {
+		// Replicate with explicit ID so replicas use the same ID.
+		RedisModuleStreamID wire_id = WIRE_STREAM_ID(stream_id);
+		RedisModule_Replicate(ctx, RDB_CMD_UPD_COMMIT, "bbb", (char *)origin->data,
+		                      origin->len, upd_txn, sizeof(*upd_txn), &wire_id, sizeof(wire_id));
 		RedisModule_ReplyWithSimpleString(ctx, RDB_RETURN_OK);
 	}
 }
@@ -2236,6 +2422,4 @@ static void upd_load(RedisModuleCtx *ctx, const arg_dname_t *origin, rdb_txn_t *
 		counter++;
 	}
 	RedisModule_ReplySetArrayLength(ctx, counter);
-
-	return;
 }
