@@ -70,6 +70,12 @@ static RedisModuleCommandArg zone_list_txt_info_args[] = {
 	{ 0 }
 };
 
+static RedisModuleCommandArg zone_info_txt_info_args[] = {
+	{"zone",     REDISMODULE_ARG_TYPE_STRING,     -1, NULL, NULL, NULL, REDISMODULE_CMD_ARG_OPTIONAL},
+	{"instance", REDISMODULE_ARG_TYPE_INTEGER,    -1, NULL, NULL, NULL, REDISMODULE_CMD_ARG_OPTIONAL},
+	{ 0 }
+};
+
 static const RedisModuleCommandInfo zone_begin_txt_info = {
 	.version = REDISMODULE_COMMAND_INFO_VERSION,
 	.summary = "Create a zone full transaction",
@@ -133,6 +139,15 @@ static const RedisModuleCommandInfo zone_list_txt_info = {
 	.args = zone_list_txt_info_args,
 };
 
+static const RedisModuleCommandInfo zone_info_txt_info = {
+	.version = REDISMODULE_COMMAND_INFO_VERSION,
+	.summary = "List zones stored in the database showing serials and updates",
+	.complexity = "O(z), where z is the number of zones",
+	.since = "7.0.0",
+	.arity = -1,
+	.args = zone_info_txt_info_args,
+};
+
 static const RedisModuleCommandInfo upd_begin_txt_info = {
 	.version = REDISMODULE_COMMAND_INFO_VERSION,
 	.summary = "Create an zone update transaction",
@@ -280,13 +295,15 @@ static int zone_commit_txt(RedisModuleCtx *ctx, RedisModuleString **argv, int ar
 	rdb_txn_t txn;
 	ARG_TXN_TXT(argv[2], txn);
 
-	zone_commit(ctx, &origin, &txn);
+	RedisModuleStreamID stream_id = { 0 };
+
+	zone_commit(ctx, &origin, &txn, &stream_id);
 	return REDISMODULE_OK;
 }
 
 static int zone_commit_bin(RedisModuleCtx *ctx, RedisModuleString **argv, int argc)
 {
-	if (argc != 3) {
+	if (argc < 3) {
 		return RedisModule_WrongArity(ctx);
 	}
 
@@ -296,7 +313,12 @@ static int zone_commit_bin(RedisModuleCtx *ctx, RedisModuleString **argv, int ar
 	rdb_txn_t txn;
 	ARG_TXN(argv[2], txn);
 
-	zone_commit(ctx, &origin, &txn);
+	RedisModuleStreamID stream_id = { 0 };
+	if (argc > 3) {
+		ARG_STREAM_ID(argv[3], stream_id);
+	}
+
+	zone_commit(ctx, &origin, &txn, &stream_id);
 	return REDISMODULE_OK;
 }
 
@@ -452,6 +474,22 @@ static int zone_list_bin(RedisModuleCtx *ctx, RedisModuleString **argv, int argc
 	return REDISMODULE_OK;
 }
 
+static int zone_info_txt(RedisModuleCtx *ctx, RedisModuleString **argv, int argc)
+{
+	arg_dname_t origin;
+	if (argc >= 2) {
+		ARG_DNAME_TXT(argv[1], origin, NULL, "zone origin");
+	}
+
+	rdb_txn_t txn;
+	if (argc >= 3) {
+		ARG_INST_TXT(argv[2], txn);
+	}
+
+	zone_info(ctx, (argc >= 2) ? &origin : NULL, (argc >= 3) ? &txn : NULL);
+	return REDISMODULE_OK;
+}
+
 static int upd_begin_txt(RedisModuleCtx *ctx, RedisModuleString **argv, int argc)
 {
 	arg_dname_t origin;
@@ -610,13 +648,15 @@ static int upd_commit_txt(RedisModuleCtx *ctx, RedisModuleString **argv, int arg
 	rdb_txn_t txn;
 	ARG_TXN_TXT(argv[2], txn);
 
-	upd_commit(ctx, &origin, &txn);
+	RedisModuleStreamID stream_id = { 0 };
+
+	upd_commit(ctx, &origin, &txn, &stream_id);
 	return REDISMODULE_OK;
 }
 
 static int upd_commit_bin(RedisModuleCtx *ctx, RedisModuleString **argv, int argc)
 {
-	if (argc != 3) {
+	if (argc < 3) {
 		return RedisModule_WrongArity(ctx);
 	}
 
@@ -626,7 +666,12 @@ static int upd_commit_bin(RedisModuleCtx *ctx, RedisModuleString **argv, int arg
 	rdb_txn_t txn;
 	ARG_TXN(argv[2], txn)
 
-	upd_commit(ctx, &origin, &txn);
+	RedisModuleStreamID stream_id = { 0 };
+	if (argc > 3) {
+		ARG_STREAM_ID(argv[3], stream_id);
+	}
+
+	upd_commit(ctx, &origin, &txn, &stream_id);
 	return REDISMODULE_OK;
 }
 
@@ -848,6 +893,7 @@ int RedisModule_OnLoad(RedisModuleCtx *ctx, RedisModuleString **argv, int argc)
 	    register_command_txt("KNOT.ZONE.LOAD",     zone_load_txt,     "readonly")   ||
 	    register_command_txt("KNOT.ZONE.PURGE",    zone_purge_txt,    "write")      ||
 	    register_command_txt("KNOT.ZONE.LIST",     zone_list_txt,     "readonly")   ||
+	    register_command_txt("KNOT.ZONE.INFO",     zone_info_txt,     "readonly")   ||
 	    register_command_txt("KNOT.UPD.BEGIN",     upd_begin_txt,     "write fast") ||
 	    register_command_txt("KNOT.UPD.ADD",       upd_add_txt,       "write fast") ||
 	    register_command_txt("KNOT.UPD.REMOVE",    upd_remove_txt,    "write fast") ||
